@@ -25,7 +25,7 @@ ALL_TESTS=""
 VERBOSE_TESTS=${V:-0}
 NUM_NS=2
 
-NEEDED_TOOLS="capinfos ethtool ip ping sed tc tcpdump timeout tshark nft socat"
+NEEDED_TOOLS="capinfos ethtool ip ping sed tc tcpdump timeout tshark nft socat ndisc6 arping"
 
 if [ -f "$TEST_CONFIG" ]; then
     source "$TEST_CONFIG"
@@ -174,19 +174,44 @@ start_background()
     local TMP_FILE="${STATEDIR}/tmp_proc_$$_$RANDOM"
     setsid bash -c "$*" &> ${TMP_FILE} &
     local PID=$!
-    sleep 2 # Wait to make sure the command is executed in the background
 
     mv "$TMP_FILE" "${STATEDIR}/proc/${PID}" >& /dev/null
 
     echo "$PID"
 }
 
+start_background_wait_output()
+{
+    local bg_func="$1"
+    local out_grep="$2"
+    local PID
+    local outfile
+    shift 2
+
+    PID=$($bg_func "$@")
+    outfile="${STATEDIR}/proc/${PID}"
+    local MAXWAIT=100
+    while ! grep -q "$out_grep" $outfile; do
+        echo "Waiting for output '$out_grep' from PID $PID">&2
+        cat $outfile >&2
+        sleep 0.1
+        MAXWAIT=$[$MAXWAIT - 1]
+        [ "$MAXWAIT" -eq 0 ] && break
+    done
+    echo "$PID"
+}
+
+start_tcpdump()
+{
+    start_background_wait_output start_background "listening on" "$@"
+}
+
+
 start_background_no_stderr()
 {
     local TMP_FILE="${STATEDIR}/tmp_proc_$$_$RANDOM"
     setsid bash -c "$*" 1> ${TMP_FILE} 2>/dev/null &
     local PID=$!
-    sleep 2 # Wait to make sure the command is executed in the background
 
     mv "$TMP_FILE" "${STATEDIR}/proc/${PID}" >& /dev/null
 
@@ -198,21 +223,37 @@ start_background_ns_devnull()
     local TMP_FILE="${STATEDIR}/tmp_proc_$$_$RANDOM"
     setsid ip netns exec "$NS" env TESTENV_NAME="$NS" "$SETUP_SCRIPT" bash -c "$*" 1>/dev/null 2>${TMP_FILE} &
     local PID=$!
-    sleep 2 # Wait to make sure the command is executed in the background
 
     mv "$TMP_FILE" "${STATEDIR}/proc/${PID}" >& /dev/null
     echo $PID
 }
 
+start_socat_ns()
+{
+    start_background_wait_output start_background_ns_devnull "listening on" "$@"
+}
+
+kill_process_group()
+{
+    local PID=$1
+
+    kill -SIGINT -$PID
+    for i in $(seq 10); do
+        ps --ppid $PID -p $PID > /dev/null || return 0
+        sleep 0.1
+    done
+    kill -TERM -$PID
+}
+
 stop_background()
 {
     local PID=$1
 
     local OUTPUT_FILE="${STATEDIR}/proc/${PID}"
-    if kill -SIGINT "-$PID" 2>/dev/null; then
-       sleep 2 # Wait to make sure the buffer is flushed after the shutdown
-       kill -SIGTERM "-$PID" 2>/dev/null && sleep 1 # just in case SIGINT was not enough
-    fi
+    local pids
+
+    kill_process_group $PID
+
 
     if [ -f "$OUTPUT_FILE" ]; then
         cat "$OUTPUT_FILE"
@@ -261,7 +302,10 @@ gen_nsname()
 iface_macaddr()
 {
     local iface="$1"
-    ip -br link show dev "$iface" | awk '{print $3}'
+    local ns="${2:-}"
+
+    [ -n "$ns" ] && ns="-n $ns"
+    ip $ns -br link show dev "$iface" | awk '{print $3}'
 }
 
 set_sysctls()
@@ -309,8 +353,9 @@ init_ns()
     ethtool -K "$nsname" rxvlan off txvlan off gro on
     ethtool -K "$peername" rxvlan off txvlan off gro on
 
-    OUTSIDE_MAC=$(iface_macaddr "$nsname")
-    INSIDE_MAC=$(iface_macaddr "$peername")
+    ip link set dev "$peername" multicast off
+    ip link set dev "$nsname" multicast off
+
     ip link set dev "$peername" netns "$nsname"
     ip link set dev "$nsname" up
     ip addr add dev "$nsname" "${OUTSIDE_IP6}/${IP6_PREFIX_SIZE}"
@@ -321,6 +366,9 @@ init_ns()
     set_sysctls veth0 "$nsname"
     ip -n "$nsname" addr add dev veth0 "${INSIDE_IP6}/${IP6_PREFIX_SIZE}"
 
+    OUTSIDE_MAC=$(iface_macaddr "$nsname")
+    INSIDE_MAC=$(iface_macaddr "veth0" "$nsname")
+
     # Prevent neighbour queries on the link
     ip neigh add "$INSIDE_IP6" lladdr "$INSIDE_MAC" dev "$nsname" nud permanent
     ip -n "$nsname" neigh add "$OUTSIDE_IP6" lladdr "$OUTSIDE_MAC" dev veth0 nud permanent
@@ -373,6 +421,8 @@ teardown()
         teardown_ns $ns
     done
 
+    [ -d "$STATEDIR" ] || return 0
+
     for f in ${STATEDIR}/proc/*; do
         if [ -f "$f" ]; then
             local pid="${f/${STATEDIR}\/proc\//}"
@@ -411,24 +461,47 @@ exec_test()
     local testn="$1"
     local output
     local ret
+    local prefix
+    local retries=${TEST_RETRIES:-1}
 
-    printf "     %-30s" "[$testn]"
+    prefix=$(printf "     %-30s" "[$testn]")
     if ! is_func "$testn"; then
-        echo "INVALID"
+        echo "${prefix}INVALID"
         return 1
     fi
 
-    output=$($testn 2>&1)
-    ret=$?
+    while [[ "$retries" -gt 0 ]]; do
+        if [ "$VERBOSE_TESTS" -eq "1" ]; then
+            echo "${prefix}START:"
+            ($testn 2>&1) | sed  's/^/          /'
+            ret=${PIPESTATUS[0]}
+            echo "          Test $testn exited with return code: $ret"
+        else
+            echo -n "$prefix"
+            output=$($testn 2>&1)
+            ret=$?
+            prefix=
+        fi
+        if [ "$ret" -eq "0" ]; then
+            break
+        else
+            retries=$[$retries - 1]
+            [ "$VERBOSE_TESTS" -eq "1" ] && echo "          Test failed - retrying $retries more times"
+            if is_func cleanup_tests; then
+                cleanup_tests || true
+            fi
+        fi
+    done
+
     if [ "$ret" -eq "0" ]; then
-        echo "PASS"
+        echo "${prefix}PASS"
     elif [ "$ret" -eq "$SKIPPED_TEST" ]; then
-        echo "SKIPPED"
+        echo "${prefix}SKIPPED"
         ret=0
     else
-        echo "FAIL"
+        echo "${prefix}FAIL"
     fi
-    if [ "$ret" -ne "0" ] || [ "$VERBOSE_TESTS" -eq "1" ]; then
+    if [ "$ret" -ne "0" ] && [ "$VERBOSE_TESTS" -ne "1" ]; then
         echo "$output" | sed  's/^/          /'
         echo "          Test $testn exited with return code: $ret"
     fi
