@@ -248,7 +248,7 @@ static int handle_ifname_multi(char *optarg, void *tgt, __unused struct prog_opt
 	if (ifindex < 0)
 		return ifindex;
 
-	iface = calloc(sizeof(*iface), 1);
+	iface = calloc(1, sizeof(*iface));
 	if (!iface)
 		return -ENOMEM;
 
@@ -456,7 +456,7 @@ static void _print_options(const struct prog_option *poptions, bool required)
 	const struct prog_option *opt;
 
 	FOR_EACH_OPTION (poptions, opt) {
-		if (opt->required != required)
+		if (opt->required != required || opt->hidden)
 			continue;
 
 		if (opt->positional) {
@@ -667,8 +667,8 @@ static int set_pos_opt(void *cfg, struct prog_option *all_opts, char *optarg)
 }
 
 int parse_cmdline_args(int argc, char **argv, struct prog_option *poptions,
-		       void *cfg, const char *prog, const char *usage_cmd,
-		       const char *doc, const void *defaults)
+		       void *cfg, size_t cfg_size, const char *prog,
+		       const char *usage_cmd, const char *doc, const void *defaults)
 {
 	struct prog_option *opt_iter;
 	struct option *long_options;
@@ -682,6 +682,9 @@ int parse_cmdline_args(int argc, char **argv, struct prog_option *poptions,
 		return -ENOMEM;
 	}
 
+	if (defaults)
+		memcpy(cfg, defaults, cfg_size);
+
 	/* Parse commands line args */
 	while ((opt = getopt_long(argc, argv, optstring,
 				  long_options, &longindex)) != -1) {
@@ -733,11 +736,6 @@ int parse_cmdline_args(int argc, char **argv, struct prog_option *poptions,
 			usage(prog, doc, poptions, full_help);
 			err = EXIT_FAILURE;
 			goto out;
-		} else if (defaults) {
-			void *dst = cfg + opt_iter->cfg_offset;
-			const void *src = defaults + opt_iter->cfg_offset;
-
-			memcpy(dst, src, opt_iter->opt_size);
 		}
 	}
 out:
@@ -783,7 +781,7 @@ int dispatch_commands(const char *argv0, int argc, char **argv,
 	if (len < 0 || (size_t)len >= sizeof(usagebuf))
 		goto out;
 
-	err = parse_cmdline_args(argc, argv, cmd->options, cfg, prog_name, usagebuf,
+	err = parse_cmdline_args(argc, argv, cmd->options, cfg, cfg_size, prog_name, usagebuf,
 				 cmd->doc, cmd->default_cfg);
 	if (err)
 		goto out;
@@ -797,11 +795,7 @@ int dispatch_commands(const char *argv0, int argc, char **argv,
 	if (err)
 		goto out;
 
-	if (prog_lock_get(prog_name))
-		goto out;
-
 	ret = cmd->func(cfg, pin_root_path);
-	prog_lock_release(0);
 out:
 	free(cfg);
 	return ret;
