@@ -71,12 +71,6 @@ static int do_basic(const struct basic_opts *opt, enum xdp_action action)
 		mask |= SAMPLE_RXQ_STATS;
 	}
 
-	if (opt->load_mode == BASIC_LOAD_BYTES && opt->program_mode != BASIC_PARSE_IPHDR) {
-		pr_warn("Setting '-l load-bytes' only works with '-p parse-ip'\n");
-		ret = EXIT_FAIL_BPF;
-		goto end_destroy;
-	}
-
 	/* Make sure we only load the one XDP program we are interested in */
 	while ((prog = bpf_object__next_program(skel->obj, prog)) != NULL)
 		if (bpf_program__type(prog) == BPF_PROG_TYPE_XDP &&
@@ -88,13 +82,13 @@ static int do_basic(const struct basic_opts *opt, enum xdp_action action)
 		opts.prog_name = "xdp_basic_prog";
 		break;
 	case BASIC_READ_DATA:
-		opts.prog_name = "xdp_read_data_prog";
+		opts.prog_name = (opt->load_mode == BASIC_LOAD_BYTES) ? "xdp_read_data_load_bytes_prog" : "xdp_read_data_prog";
 		break;
 	case BASIC_PARSE_IPHDR:
 		opts.prog_name = (opt->load_mode == BASIC_LOAD_BYTES) ? "xdp_parse_load_bytes_prog" : "xdp_parse_prog";
 		break;
 	case BASIC_SWAP_MACS:
-		opts.prog_name = "xdp_swap_macs_prog";
+		opts.prog_name = (opt->load_mode == BASIC_LOAD_BYTES) ? "xdp_swap_macs_load_bytes_prog" : "xdp_swap_macs_prog";
 		break;
 	}
 
@@ -137,7 +131,7 @@ static int do_basic(const struct basic_opts *opt, enum xdp_action action)
 	ret = EXIT_FAIL;
 
 	pr_info("%s packets on %s (ifindex %d; driver %s)\n",
-		action == XDP_DROP ? "Dropping" : "Hairpinning (XDP_TX)",
+		action == XDP_DROP ? "Dropping" : action == XDP_TX ? "Hairpinning (XDP_TX)" : "Passing",
 		opt->iface_in.ifname, opt->iface_in.ifindex, get_driver_name(opt->iface_in.ifindex));
 
 	ret = sample_run(opt->interval, NULL, NULL);
