@@ -670,6 +670,7 @@ static bool capture_on_legacy_interface(struct dumpopt *cfg)
 	}
 	exit_pcap = NULL;
 	rc = true;
+	fflush(stdout);
 
 	fprintf(stderr, "\n%"PRIu64" packets captured\n", captured_packets);
 	if (pcap_stats(pcap, &ps) == 0) {
@@ -1767,19 +1768,6 @@ static bool capture_on_interface(struct dumpopt *cfg)
 		}
 	}
 
-	/* No more error conditions, display some capture information */
-	fprintf(stderr, "listening on %s, ingress XDP program ",
-		cfg->iface.ifname);
-
-	for (unsigned int i = 0; i < tgt_progs.nr_of_progs; i++)
-		fprintf(stderr, "ID %u func %s, ",
-			xdp_program__id(tgt_progs.progs[i].prog),
-			tgt_progs.progs[i].func);
-
-	fprintf(stderr, "capture mode %s, capture size %d bytes\n",
-		get_capture_mode_string(tgt_progs.progs[0].rx_capture),
-		cfg->snaplen);
-
 	/* Setup perf context */
 	memset(&perf_ctx, 0, sizeof(perf_ctx));
 	perf_ctx.cfg = cfg;
@@ -1846,6 +1834,20 @@ static bool capture_on_interface(struct dumpopt *cfg)
 		goto error_exit;
 	}
 
+	/* No more error conditions, display some capture information */
+	fprintf(stderr, "listening on %s, ingress XDP program ",
+		cfg->iface.ifname);
+
+	for (unsigned int i = 0; i < tgt_progs.nr_of_progs; i++)
+		fprintf(stderr, "ID %u func %s, ",
+			xdp_program__id(tgt_progs.progs[i].prog),
+			tgt_progs.progs[i].func);
+
+	fprintf(stderr, "capture mode %s, capture size %d bytes\n",
+		get_capture_mode_string(tgt_progs.progs[0].rx_capture),
+		cfg->snaplen);
+	fflush(stderr);
+
 	/* Loop trough the dumper */
 	while (!exit_xdpdump) {
 		cnt = perf_buffer__poll(perf_buf, 1000);
@@ -1859,6 +1861,7 @@ static bool capture_on_interface(struct dumpopt *cfg)
 	perf_buffer__consume(perf_buf);
 #endif
 
+	fflush(stdout);
 	fprintf(stderr, "\n%"PRIu64" packets captured\n",
 		perf_ctx.captured_packets);
 	fprintf(stderr, "%"PRIu64" packets dropped by perf ring\n",
