@@ -6,13 +6,7 @@ PIDS=""
 
 skip_if_missing_kernel_support()
 {
-    $XDP_TRAFFICGEN probe
-    ret=$?
-    if [ "$ret" -eq "161" ]; then
-        exit $SKIPPED_TEST
-    elif [ "$ret" -ne "0" ]; then
-        exit 1
-    fi
+    $XDP_TRAFFICGEN probe || exit $SKIPPED_TEST
 }
 
 skip_if_missing_kernel_features()
