@@ -71,6 +71,7 @@ static const char *driver_pass_list[] = {
 static bool driver_needs_xdp_pass(const struct iface *iface)
 {
 	const char *name = get_driver_name(iface->ifindex);
+	struct xdp_multiprog *mp;
 	__u64 feature_flags;
 	size_t i;
 	int err;
@@ -80,6 +81,13 @@ static bool driver_needs_xdp_pass(const struct iface *iface)
 	if (!err && feature_flags & NETDEV_XDP_ACT_NDO_XMIT)
 		return false;
 
+	mp = xdp_multiprog__get_from_ifindex(iface->ifindex);
+	if (!IS_ERR_OR_NULL(mp)) {
+		pr_debug("Interface %s already has an XDP program loaded\n", iface->ifname);
+		xdp_multiprog__close(mp);
+		return false;
+	}
+
 	for (i = 0; i < ARRAY_SIZE(driver_pass_list); i++) {
 		if (!strcmp(name, driver_pass_list[i])) {
 			pr_debug("Driver %s on interface %s needs an xdp_pass program to use XDP_REDIRECT\n",
@@ -108,8 +116,7 @@ static int check_iface_support(const struct iface *iface)
 			"Continuing anyway, but running may fail!\n",
 			iface->ifname, -err);
 	} else if (!(feature_flags & NETDEV_XDP_ACT_NDO_XMIT)) {
-		pr_warn("Interface %s does not support sending packets via XDP.\n"
-			"Most likely the driver is missing support in this kernel version.\n",
+		pr_warn("Interface %s does not support sending packets via XDP.\n",
 			iface->ifname);
 		return -EOPNOTSUPP;
 	}
@@ -523,8 +530,10 @@ int do_udp(const void *opt, __unused const char *pin_root_path)
 	}
 
 	if (driver_needs_xdp_pass(&cfg->iface)) {
-		opts.prog_name = "xdp_pass";
-		pass_prog = xdp_program__create(&opts);
+		DECLARE_LIBXDP_OPTS(xdp_program_opts, pass_opts);
+		pass_opts.prog_name = "xdp_pass";
+		pass_opts.find_filename = "xdp-dispatcher.o";
+		pass_prog = xdp_program__create(&pass_opts);
 		if (!pass_prog) {
 			err = -errno;
 			pr_warn("Couldn't load xdp_pass program\n");
@@ -540,7 +549,7 @@ int do_udp(const void *opt, __unused const char *pin_root_path)
 		err = xdp_program__attach(pass_prog, cfg->iface.ifindex,
 					  XDP_MODE_NATIVE, 0);
 		if (err) {
-			pr_warn("Couldn't attach xdp_pass program");
+			pr_warn("Couldn't attach xdp_pass program\n");
 			xdp_program__close(pass_prog);
 			pass_prog = NULL;
 			goto out;
@@ -1031,20 +1040,37 @@ static struct prog_option probe_options[] = {
 		      .metavar = "<ifname>",
 		      .short_opt = 'i',
 		      .help = "Probe features of device <ifname>"),
+	END_OPTIONS
 };
 
 int do_probe(const void *opt, __unused const char *pin_root_path)
 {
 	const struct probeopt *cfg = opt;
-	int err;
+	int err1 = 0, err2;
 
-	if (cfg->iface.ifindex)
-		check_iface_support(&cfg->iface);
+	if (cfg->iface.ifindex) {
+		err1 = check_iface_support(&cfg->iface);
+		if (err1) {
+			const char *name = get_driver_name(cfg->iface.ifindex);
+			if (driver_needs_xdp_pass(&cfg->iface)) {
+				pr_info(" Note that this driver (%s) needs an XDP program "
+					"loaded to use XDP_REDIRECT.\n"
+					" Loading a dummy XDP program on the interface "
+					"may enable support.\n", name);
+			} else {
 
-	err = probe_kernel_support();
-	if (!err)
+				if (!strcmp(name, "veth"))
+					pr_info(" Note that enabling GRO on both ends of a "
+						"veth pair may enable XDP support\n");
+			}
+		}
+	}
+
+	err2 = probe_kernel_support();
+	if (!err2)
 		pr_info("Kernel supports live packet mode for XDP BPF_PROG_RUN.\n");
-	return err;
+
+	return !(!err1 && !err2);
 }
 
 int do_help(__unused const void *cfg, __unused const char *pin_root_path)
@@ -1055,6 +1081,7 @@ int do_help(__unused const void *cfg, __unused const char *pin_root_path)
 		"COMMAND can be one of:\n"
 		"       udp         - run in UDP mode\n"
 		"       tcp         - run in TCP mode\n"
+		"       probe       - probe kernel support\n"
 		"       help        - show this help message\n"
 		"\n"
 		"Use 'xdp-trafficgen COMMAND --help' to see options for each command\n");
